/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock.junit.configuration.component;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanException;
import javax.management.ReflectionException;
import javax.management.StandardMBean;

import org.ow2.petals.jmx.api.api.configuration.component.RuntimeConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.configuration.component.exception.ConfigurationComponentErrorException;
import org.ow2.petals.jmx.api.api.configuration.component.exception.RuntimeConfigurationComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.AttributeErrorException;

/**
 * An abstract class for mocks: {@link RuntimeConfigurationComponentClient} and
 * {@link InstallationConfigurationServiceClientMock}
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public abstract class AbstractConfigurationServiceClientMock implements RuntimeConfigurationComponentClient {

    private StandardMBean mbean;

    public AbstractConfigurationServiceClientMock() {
        this.mbean = null;
    }

    public AbstractConfigurationServiceClientMock(StandardMBean mbean) {
        this.mbean = mbean;
    }

    @Override
    public Map<MBeanAttributeInfo, Object> getConfigurationMBeanAttributes()
            throws RuntimeConfigurationComponentErrorException {
        if (this.mbean != null) {
            Map<MBeanAttributeInfo, Object> res = new HashMap<>();
            for (MBeanAttributeInfo attribute : this.mbean.getMBeanInfo().getAttributes()) {
                try {
                    res.put(attribute, this.mbean.getAttribute(attribute.getName()));
                } catch (AttributeNotFoundException | MBeanException | ReflectionException e) {
                    throw new RuntimeConfigurationComponentErrorException(e);
                }
            }
            return res;
        } else {
            return Collections.emptyMap();
        }
    }

    @Override
    public void setConfigurationMBeanAttributes(final Map<MBeanAttributeInfo, Object> attributes)
            throws ConfigurationComponentErrorException {
        if (this.mbean != null) {
            final AttributeList attributesList = new AttributeList();
            for (final Entry<MBeanAttributeInfo, Object> entry : attributes.entrySet()) {
                attributesList.add(new Attribute(entry.getKey().getName(), entry.getValue()));
            }
            this.mbean.setAttributes(attributesList);
        }
    }

    @Override
    public Object getAttribute(final String attribute) throws AttributeErrorException {
        if (this.mbean != null) {
            try {
                return this.mbean.getAttribute(attribute);
            } catch (AttributeNotFoundException | MBeanException | ReflectionException e) {
                throw new AttributeErrorException(e);
            }
        } else {
            throw new AttributeErrorException(new AttributeNotFoundException());
        }
    }

    @Override
    public void setAttribute(final String name, final Object value) throws AttributeErrorException {
        if (this.mbean != null) {
            try {
                this.mbean.setAttribute(new Attribute(name, value));
            } catch (AttributeNotFoundException | InvalidAttributeValueException | MBeanException
                    | ReflectionException e) {
                throw new AttributeErrorException(e);
            }
        } else {
            throw new AttributeErrorException(new AttributeNotFoundException());
        }
    }

}